/* analog_display
 *
 *  Egy analóg hőmérő jelét mérjük az A0 bemeneten és
 * az eredményt átszámítás után kiíratjuk a
 * 2x 8 digites, PCF8577C vezérlőjű LCD kijelzőn.
 *
 * Kapcsolás:
 *              +5V  +5V
 *               |    |
 *   ---------- 4.7k 4.7k  -----------------------------------------------~
 *  | Arduino  | |    |   | PCF8577C LCD                                  ~
 *  |       A4 |-+----|---| SDA     _    _    _    _    _    _    _    _  ~
 *  |       A5 |------+---| SCL   /_ / /_ / /_ / /_ / /_ / /_ / /_ / /_ / ~
 *  |      GND |----------| GND  /_ / /_ / /_ / /_ / /_ / /_ / /_ / /_ /  ~
 *  |       5V |-------+--| VCC                                           ~
 *  |       A0 |----+  |   -----------------------------------------------~
 *   ----------     |  |    _____
 *                  |  +---| VCC |
 *                  +------| OUT |  MCP9700A
 *                     +---| GND |  analog thermometer
 *                     |    -----
 *                    ---
 */
#include <Wire.h>
#define SLAVE_ADDRESS 0x3A
byte digits[10] = {126, 18, 188, 182, 210, 230, 238, 50, 254, 246};
byte data[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // Display buffer

void setup() {
  Wire.begin();                 // Az I2C csatorna inicializálása
  analogReference(INTERNAL);    // Belső 1,1 V-os referencia
}

void loop() {
  long sum = 0;
  for(int i=0; i<256; i++) sum += analogRead(A0);
  long a0 = sum>>8;              // ADC adat (0-1023)
  long mv = a0 * 1100 / 1024;    // millivolts (0-1100)
  long temp10 = mv - 500;        // tempC*10 (-500 - 600)
  clrScreen();
  outdec(a0,0,12);
  outdec(mv,0,8);
  outdec(temp10,1,2);
  data[1] = 0xF0;                // fok jel
  data[0] = 0x6C;                // C betű
  //--- A kijelző frissítése -----------
  Wire.beginTransmission(SLAVE_ADDRESS);
  Wire.write(0);              // regsiztercím megadása (ide írunk)
  Wire.write(data, 16);       // A 16 adat kiírása egy tranzakcióban
  Wire.endTransmission();     // Kiírás vége
  delay(500);                 // Fél másodperc várakozás
}

void clrScreen(void) {
  for(int i=0; i<16; data[i++]=0);
}

/** Decimális kiíratás adott számú tizedesjegyre.
   num a kiírandó szám (előjelesen)
   ndigits a kiírandó tizedesek száma
*/
void outdec(long num, byte ndigits, byte p) {
  unsigned int i, sign, dp;
  i = 0; sign = 0; dp = 0;
  if (num < 0) {
    sign = 128;
    num = -num;
  }
  do {
    data[p + i++] = digits[num % 10] | dp;
    num = num / 10;
    if (i == ndigits) dp = 1;
    else dp = 0;
  } while (num > 0);
  data[p + i++] = sign;
}
